@@ -32,8 +32,9 @@ import socket
 from abc import ABC
 from typing import Optional, Iterable
 
-from startrek.types import Address, AddressPairMap
-from startrek import Channel
+from startrek.types import SocketAddress, AddressPairMap
+from startrek.fsm import Runner
+from startrek import Channel, BaseChannel
 from startrek import Connection, ConnectionDelegate
 from startrek import BaseConnection, ActiveConnection
 from startrek import BaseHub
@@ -44,7 +45,7 @@ from .channel import PacketChannel
 class ChannelPool(AddressPairMap[Channel]):
 
     # Override
-    def get(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
+    def get(self, remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
         assert not (remote is None and local is None), 'both addresses are empty'
         # -- Step 1 --
         #     (remote, local)
@@ -85,21 +86,29 @@ class ChannelPool(AddressPairMap[Channel]):
         # not found
 
     # Override
-    def set(self, remote: Optional[Address], local: Optional[Address], item: Optional[Channel]):
-        old = self.get(remote=remote, local=local)
-        if old is not None and old is not item:
-            self.remove(remote=remote, local=local, item=old)
-        super().set(remote=remote, local=local, item=item)
+    def set(self, item: Optional[Channel],
+            remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        # 1. remove cached item
+        cached = super().remove(item=item, remote=remote, local=local)
+        if cached is not None and cached is not item:
+            Runner.async_run(coroutine=cached.close())
+        # 2. set new item
+        old = super().set(item=item, remote=remote, local=local)
+        assert old is None, 'should not happen: %s' % old
+        return cached
 
     # Override
-    def remove(self, remote: Optional[Address], local: Optional[Address], item: Optional[Channel]) -> Optional[Channel]:
-        cached = super().remove(remote=remote, local=local, item=item)
-        if cached is not None:
-            if not cached.closed:
-                cached.close()
-            return cached
+    def remove(self, item: Optional[Channel],
+               remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        cached = super().remove(item=item, remote=remote, local=local)
+        if cached is not None and cached is not item:
+            Runner.async_run(coroutine=cached.close())
+        if item is not None:
+            Runner.async_run(coroutine=item.close())
+        return cached
 
 
+# noinspection PyAbstractClass
 class PacketHub(BaseHub, ABC):
     """ Base Datagram Hub """
 
@@ -111,28 +120,31 @@ class PacketHub(BaseHub, ABC):
     def _create_channel_pool(self):
         return ChannelPool()
 
-    def bind(self, address: Address = None, host: str = None, port: int = 0):
+    async def bind(self, address: SocketAddress = None, host: str = None, port: int = 0):
         if address is None:
-            assert port > 0, 'address error: (%s:%d)' % (host, port)
+            assert host is not None and port > 0, 'address error: (%s:%d)' % (host, port)
             address = (host, port)
-        channel = self._get_channel(remote=None, local=address)
+        channel = self.__channel_pool.get(remote=None, local=address)
         if channel is None:
+            channel = self._create_channel(remote=None, local=address)
+            assert isinstance(channel, BaseChannel), 'channel error: %s, %s' % (address, channel)
             sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
             sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
             sock.setblocking(True)
             sock.bind(address)
             sock.setblocking(False)
-            channel = self._create_channel(remote=None, local=address, sock=sock)
-            self._set_channel(remote=None, local=address, channel=channel)
+            # set socket for this channel
+            await channel.set_socket(sock=sock)
+            self.__channel_pool.set(item=channel, remote=None, local=address)
 
     #
     #   Channel
     #
 
     # noinspection PyMethodMayBeStatic
-    def _create_channel(self, remote: Optional[Address], local: Optional[Address], sock: socket.socket) -> Channel:
+    def _create_channel(self, remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Channel:
         # override for user-customized channel
-        return PacketChannel(remote=remote, local=local, sock=sock)
+        return PacketChannel(remote=remote, local=local)
 
     # Override
     def _all_channels(self) -> Iterable[Channel]:
@@ -140,41 +152,46 @@ class PacketHub(BaseHub, ABC):
         return self.__channel_pool.items
 
     # Override
-    def _remove_channel(self, remote: Optional[Address], local: Optional[Address], channel: Optional[Channel]):
+    def _remove_channel(self, channel: Optional[Channel],
+                        remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
         """ remove cached channel """
-        self.__channel_pool.remove(remote=remote, local=local, item=channel)
+        return self.__channel_pool.remove(item=channel, remote=remote, local=local)
 
-    def _get_channel(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
+    def _get_channel(self, remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
         """ get cached channel """
         return self.__channel_pool.get(remote=remote, local=local)
 
-    def _set_channel(self, remote: Optional[Address], local: Optional[Address], channel: Channel):
+    def _set_channel(self, channel: Channel,
+                     remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
         """ cache channel """
-        self.__channel_pool.set(remote=remote, local=local, item=channel)
-
-    # Override
-    def open(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
-        # get channel with direction (remote, local)
-        return self._get_channel(remote=remote, local=local)
+        return self.__channel_pool.set(item=channel, remote=remote, local=local)
 
 
 class ServerHub(PacketHub):
     """ Datagram Server Hub """
 
     # Override
-    def _create_connection(self, remote: Address, local: Optional[Address], channel: Channel) -> Optional[Connection]:
-        conn = BaseConnection(remote=remote, local=local, channel=channel)
+    def _create_connection(self, remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Connection]:
+        conn = BaseConnection(remote=remote, local=local)
         conn.delegate = self.delegate  # gate
-        conn.start()  # start FSM
         return conn
 
+    # Override
+    async def open(self, remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        # get channel with direction (remote, local)
+        return self._get_channel(remote=remote, local=local)
+
 
 class ClientHub(PacketHub):
     """ Datagram Client Hub """
 
     # Override
-    def _create_connection(self, remote: Address, local: Optional[Address], channel: Channel) -> Optional[Connection]:
-        conn = ActiveConnection(remote=remote, local=None, channel=channel, hub=self)
+    def _create_connection(self, remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Connection]:
+        conn = ActiveConnection(remote=remote, local=local)
         conn.delegate = self.delegate  # gate
-        conn.start()  # start FSM
         return conn
+
+    # Override
+    async def open(self, remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        # get channel with direction (remote, local)
+        return self._get_channel(remote=remote, local=local)
