@@ -66,26 +66,6 @@ class PackageArrival(ArrivalShip):
     def sn(self) -> TransactionID:
         return self.__head.sn
 
-    # Override
-    def __eq__(self, other) -> bool:
-        if isinstance(other, PackageArrival):
-            if other is self:
-                return True
-            return self.__head == other.__head and self.__body == other.__body
-
-    # Override
-    def __ne__(self, other) -> bool:
-        if isinstance(other, PackageArrival):
-            if other is self:
-                return False
-            return self.__head != other.__head or self.__body != other.__body
-        else:
-            return True
-
-    # Override
-    def __hash__(self) -> int:
-        return hash(self.__head) * 13 + hash(self.__body)
-
     # Override
     def assemble(self, ship):  # -> Optional[PackageArrival]:
         if self.__completed is None and ship is not self:
@@ -129,26 +109,6 @@ class PackageDeparture(DepartureShip):
     def sn(self) -> TransactionID:
         return self.__head.sn
 
-    # Override
-    def __eq__(self, other) -> bool:
-        if isinstance(other, PackageDeparture):
-            if other is self:
-                return True
-            return self.__head == other.__head and self.__body == other.__body
-
-    # Override
-    def __ne__(self, other) -> bool:
-        if isinstance(other, PackageDeparture):
-            if other is self:
-                return False
-            return self.__head != other.__head or self.__body != other.__body
-        else:
-            return True
-
-    # Override
-    def __hash__(self) -> int:
-        return hash(self.__head) * 13 + hash(self.__body)
-
     @property  # Override
     def fragments(self) -> List[bytes]:
         if len(self.__fragments) == 0 and len(self.__packages) > 0:
@@ -194,7 +154,7 @@ class PackageDeparture(DepartureShip):
 class PackageDocker(StarDocker):
 
     # noinspection PyMethodMayBeStatic
-    def _parse_package(self, data: Optional[bytes]) -> Optional[Package]:
+    def _parse_package(self, data: bytes) -> Optional[Package]:
         if data is not None:  # and len(data) > 0:
             return Package.parse(data=Data(buffer=data))
 
@@ -214,13 +174,15 @@ class PackageDocker(StarDocker):
             return PackageDeparture(pack=pack, priority=priority, max_tries=1)
 
     # Override
-    def _get_arrival(self, data: bytes) -> Optional[Arrival]:
+    def _get_arrivals(self, data: bytes) -> List[Arrival]:
         pack = self._parse_package(data=data)
-        if pack is not None:  # and pack.body.size > 0:
-            return self._create_arrival(pack=pack)
+        if pack is None:
+            return []
+        else:
+            return [self._create_arrival(pack=pack)]
 
     # Override
-    def _check_arrival(self, ship: Arrival) -> Optional[Arrival]:
+    async def _check_arrival(self, ship: Arrival) -> Optional[Arrival]:
         assert isinstance(ship, PackageArrival), 'arrival ship error: %s' % ship
         pack = ship.package
         if pack is None:
@@ -237,7 +199,7 @@ class PackageDocker(StarDocker):
             # process CommandResponse
             #       'PONG'
             #       'OK'
-            self._check_response(ship=ship)
+            await self._check_response(ship=ship)
             if body == PONG or body == OK:
                 # command responded
                 return None
@@ -249,11 +211,11 @@ class PackageDocker(StarDocker):
             #       '...'
             if body == PING:
                 # PING -> PONG
-                self._respond_command(sn=head.sn, body=PONG)
+                await self._respond_command(sn=head.sn, body=PONG)
                 return None
             else:
                 # respond for Command
-                self._respond_command(sn=head.sn, body=OK)
+                await self._respond_command(sn=head.sn, body=OK)
             # Unknown Command?
             # let the caller to process it
         elif data_type.is_message_response:
@@ -263,7 +225,7 @@ class PackageDocker(StarDocker):
             if body == AGAIN:
                 # TODO: reset retries?
                 return None
-            self._check_response(ship=ship)
+            await self._check_response(ship=ship)
             if body == OK:
                 # message responded
                 return None
@@ -271,7 +233,7 @@ class PackageDocker(StarDocker):
             # let the caller to process it
         else:
             # respond for Message/Fragment
-            self._respond_message(sn=head.sn, pages=head.pages, index=head.index)
+            await self._respond_message(sn=head.sn, pages=head.pages, index=head.index)
             if data_type.is_message_fragment:
                 # assemble MessageFragment with cached fragments to completed Message
                 # let the caller to process the completed message
@@ -290,7 +252,7 @@ class PackageDocker(StarDocker):
         return ship
 
     #
-    #   Sending
+    #   Packing
     #
 
     # noinspection PyMethodMayBeStatic
@@ -309,36 +271,40 @@ class PackageDocker(StarDocker):
     def _create_message_response(self, sn: TransactionID, pages: int, index: int) -> Package:
         return Package.new(data_type=DataType.MESSAGE_RESPONSE, sn=sn, pages=pages, index=index, body=Data(buffer=OK))
 
+    #
+    #   Sending
+    #
+
     # protected
-    def _respond_command(self, sn: TransactionID, body: bytes):
+    async def _respond_command(self, sn: TransactionID, body: bytes) -> bool:
         pack = self._create_command_response(sn=sn, body=body)
-        self.send_package(pack=pack)
+        return await self.send_package(pack=pack)
 
     # protected
-    def _respond_message(self, sn: TransactionID, pages: int, index: int):
+    async def _respond_message(self, sn: TransactionID, pages: int, index: int) -> bool:
         pack = self._create_message_response(sn=sn, pages=pages, index=index)
-        self.send_package(pack=pack)
+        return await self.send_package(pack=pack)
 
-    def send_command(self, body: Union[bytes, bytearray]) -> bool:
+    async def send_command(self, body: Union[bytes, bytearray]) -> bool:
         pack = self._create_command(body=body)
-        return self.send_package(pack=pack, priority=DeparturePriority.SLOWER)
+        return await self.send_package(pack=pack, priority=DeparturePriority.SLOWER)
 
-    def send_message(self, body: Union[bytes, bytearray]) -> bool:
+    async def send_message(self, body: Union[bytes, bytearray]) -> bool:
         pack = self._create_message(body=body)
-        return self.send_package(pack=pack, priority=DeparturePriority.NORMAL)
+        return await self.send_package(pack=pack, priority=DeparturePriority.NORMAL)
 
-    def send_package(self, pack: Package, priority: int = 0) -> bool:
+    async def send_package(self, pack: Package, priority: int = 0) -> bool:
         """ send data package with priority """
         outgo = self._create_departure(pack=pack, priority=priority)
-        return self.send_ship(ship=outgo)
+        return await self.send_ship(ship=outgo)
 
     # Override
-    def send_data(self, payload: Union[bytes, bytearray]) -> bool:
-        return self.send_message(body=payload)
+    async def send_data(self, payload: Union[bytes, bytearray]) -> bool:
+        return await self.send_message(body=payload)
 
     # Override
-    def heartbeat(self):
-        self.send_command(body=PING)
+    async def heartbeat(self):
+        await self.send_command(body=PING)
 
 
 PING = b'PING'
